@@ -6,7 +6,7 @@
 %%% Created : 24 Nov 2015 by Evgeny Khramtsov <ekhramtsov@process-one.net>
 %%%
 %%%
-%%% Copyright (C) 2002-2018 ProcessOne, SARL. All Rights Reserved.
+%%% Copyright (C) 2002-2019 ProcessOne, SARL. All Rights Reserved.
 %%%
 %%% Licensed under the Apache License, Version 2.0 (the "License");
 %%% you may not use this file except in compliance with the License.
@@ -206,8 +206,8 @@ resourceprep(S) when byte_size(S) < 1024 ->
     end;
 resourceprep(_) -> error.
 
--spec tolower(jid() | ljid()) -> error | ljid().
-
+-spec tolower(jid()) -> ljid();
+	     (ljid()) -> error | ljid().
 tolower(#jid{luser = U, lserver = S,
 	     lresource = R}) ->
     {U, S, R};
