@@ -5,7 +5,7 @@
 %%% @end
 %%%
 %%%
-%%% Copyright (C) 2002-2018 ProcessOne, SARL. All Rights Reserved.
+%%% Copyright (C) 2002-2019 ProcessOne, SARL. All Rights Reserved.
 %%%
 %%% Licensed under the Apache License, Version 2.0 (the "License");
 %%% you may not use this file except in compliance with the License.
@@ -27,7 +27,8 @@
 -export([compile/1, compile/2]).
 -export([dec_int/1, dec_int/3, dec_enum/2, dec_bool/1, not_empty/1,
 	 dec_enum_int/2, dec_enum_int/4, enc_int/1, enc_enum/1,
-	 enc_bool/1, enc_enum_int/1, format_error/1, io_format_error/1]).
+	 enc_bool/1, enc_enum_int/1, dec_ip/1, enc_ip/1,
+	 format_error/1, io_format_error/1]).
 -include("xmpp.hrl").
 
 -type mfargs() :: {atom(), atom(), list()} | {atom(), list()}.
@@ -44,6 +45,7 @@
 		dec_mfas = [] :: [{binary(), mfargs()}],
 		enc_mfas = [] :: [{binary(), mfargs()}],
 		specs = [] :: [{binary(), string()}],
+		ignore_unknown = false :: boolean(),
 		required = [] :: [{binary(), boolean()} | binary()],
 		defaults = [] :: [{binary(), any()}]}).
 
@@ -97,6 +99,7 @@ compile_file(Path, Opts) ->
 		       dec_mfas = proplists:get_value(decode, Config, []),
 		       enc_mfas = proplists:get_value(encode, Config, []),
 		       specs = proplists:get_value(specs, Config, []),
+		       ignore_unknown = proplists:get_bool(ignore_unknown, Config),
 		       required = proplists:get_value(required, Config, []),
 		       defaults = proplists:get_value(defaults, Config, [])},
 	#xmlel{} = El = fxml_stream:parse_element(Data),
@@ -167,6 +170,16 @@ enc_bool(false) -> <<"0">>.
 not_empty(<<_, _/binary>> = Val) ->
     Val.
 
+dec_ip(Val) ->
+    {ok, Addr} = inet_parse:address(binary_to_list(Val)),
+    Addr.
+
+enc_ip({0,0,0,0,0,16#ffff,A,B}) ->
+    enc_ip({(A bsr 8) band 16#ff, A band 16#ff,
+            (B bsr 8) band 16#ff, B band 16#ff});
+enc_ip(Addr) ->
+    list_to_binary(inet_parse:ntoa(Addr)).
+
 format_error({form_type_mismatch, Type}) ->
     <<"FORM_TYPE doesn't match '", Type/binary, "'">>;
 format_error({bad_var_value, Var, Type}) ->
@@ -217,6 +230,7 @@ compile_element(#xmlel{name = <<"form_type">>, children = Els} = Form,
 	#xdata{fields = Fs} = xmpp_codec:decode(X),
 	put(outbuf, []),
 	mk_header(State),
+	mk_specs(Fs, State),
 	mk_aux_funs(),
 	mk_top_decoder(Fs, State),
 	mk_top_encoder(Fs, State),
@@ -224,7 +238,8 @@ compile_element(#xmlel{name = <<"form_type">>, children = Els} = Form,
 	mk_encoders(Fs, State),
 	ErlData = get(outbuf),
 	ok = file:write_file(filename:join(ErlDir, OutErl), ErlData),
-	ok = erl_tidy:file(filename:join(ErlDir, OutErl), [{backups, false}]),
+	ok = erl_tidy:file(filename:join(ErlDir, OutErl),
+			   [{backups, false}, {keep_unused, true}]),
 	put(outbuf, []),
 	mk_type_definitions(Fs, State),
 	HrlData = get(outbuf),
@@ -238,21 +253,12 @@ mk_aux_funs() ->
         {ok, AbsCode} ->
             AST = lists:filter(
 		    fun(T) ->
-			    case catch erl_syntax_lib:analyze_function(T) of
+			    try erl_syntax_lib:analyze_function(T) of
 				{format_error, 1} -> true;
 				{io_format_error, 1} -> true;
-				{dec_int, 3} -> true;
-				{dec_int, 1} -> true;
-				{dec_enum, 2} -> true;
-				{dec_enum_int, 2} -> true;
-				{dec_enum_int, 4} -> true;
-				{enc_int, 1} -> true;
-				{enc_enum, 1} -> true;
-				{enc_enum_int, 1} -> true;
-				{not_empty, 1} -> true;
-				{dec_bool, 1} -> true;
-				{enc_bool, 1} -> true;
-				_ -> false
+				FA -> lists:member(FA, codec_funs())
+			    catch _:_ ->
+				    false
 			    end
 		    end, AbsCode),
 	    emit_raw(erl_prettypr:format(erl_syntax:form_list(AST)) ++ io_lib:nl());
@@ -286,23 +292,37 @@ mk_comment_header(#state{file_name = Source, xmlns = NS, doc = Doc}) ->
 mk_header(#state{mod_name = Mod, hrl = Include, xmlns = NS} = State) ->
     mk_comment_header(State),
     emit("~n-module(~s).~n", [Mod]),
+    emit("-compile({nowarn_unused_function, ~p}).~n", [codec_funs()]),
+    emit("-compile(nowarn_unused_vars).~n"),
+    emit("-dialyzer({nowarn_function, dec_int/3}).~n"),
     case NS of
-	[_] -> emit("-export([encode/1, encode/2]).~n");
-	_ -> emit("-export([encode/2, encode/3]).~n")
+	[_] -> emit("-export([encode/1, encode/2, encode/3]).~n");
+	_ -> emit("-export([encode/2, encode/3, encode/4]).~n")
     end,
-    emit("-export([decode/1, decode/2, format_error/1, io_format_error/1]).~n"),
+    emit("-export([decode/1, decode/2, decode/3, format_error/1, io_format_error/1]).~n"),
     emit("-include(\"xmpp_codec.hrl\").~n"),
     emit("-include(\"~s\").~n", [Include]),
-    emit("-export_type([property/0, result/0, form/0]).~n"),
-    case lists:any(
-	   fun({_, {dec_int, _}}) -> true;
-	      (_) -> false
-	   end, State#state.dec_mfas) of
-	true ->
-	    emit("-dialyzer({nowarn_function, dec_int/3}).~n");
-	false ->
-	    ok
-    end.
+    emit("-export_type([property/0, result/0, form/0, error_reason/0]).~n"),
+    emit("-define(T(S), <<S>>).~n").
+
+mk_specs(Fs, State) ->
+    emit("-spec format_error(error_reason()) -> binary().~n"),
+    emit("-spec io_format_error(error_reason()) -> {binary(), [binary()]}.~n"),
+    emit("-spec decode([xdata_field()]) -> result().~n"),
+    emit("-spec decode([xdata_field()], [binary(), ...]) -> result().~n"),
+    emit("-spec decode([xdata_field()], [binary(), ...], [binary()]) -> result().~n"),
+    emit("-spec decode([xdata_field()], [binary(), ...], [binary()], result()) -> result().~n"),
+    emit("-spec do_decode([xdata_field()], binary(), [binary()], result()) -> result().~n"),
+    BinType = case State#state.xmlns of
+		  [_] -> "";
+		  _ -> ", binary()"
+	      end,
+    Required = [io_lib:format("'~s'", [var_to_rec_field(Var, State)])
+		|| #xdata_field{var = Var} <- Fs],
+    emit("-spec encode(form()~s) -> [xdata_field()].~n", [BinType]),
+    emit("-spec encode(form()~s, binary()) -> [xdata_field()].~n", [BinType]),
+    emit("-spec encode(form()~s, binary(), [~s]) -> [xdata_field()].~n",
+	 [BinType, string:join(Required, " | ")]).
 
 mk_type_definitions(Fs, State) ->
     mk_comment_header(State),
@@ -326,74 +346,94 @@ mk_type_definitions(Fs, State) ->
 	       end, Fs),
     emit(string:join(Fields, " |~n                    ") ++ ".~n"),
     emit("-type result() :: [property()].~n~n"),
-    VarsWithSpec = lists:flatmap(
-		     fun(#xdata_field{type = T, var = Var} = F)
-			   when ?is_list_type(T) ->
-			     RecName = var_to_rec_field(Var, State),
-			     Spec0 = get_typespec(F, State),
-			     Spec = case is_complex_type(Spec0) of
-					true ->
-					    io_lib:format("'~s'()", [RecName]);
-					false ->
-					    Spec0
-				    end,
-			     [{RecName, mk_typespec(F, State), Spec}];
-			(_) ->
-			     []
-		     end, Fs),
-    case VarsWithSpec of
-	[] ->
-	    emit("-type form() :: [property() | xdata_field()].~n");
-	_ ->
-	    emit("-type options(T) :: [{binary(), T}].~n"),
-	    emit("-type property_with_options() ::~n      "),
-	    Options = [io_lib:format("{'~s', ~s, options(~s)}",
-				     [Var, Spec1, Spec2])
-		       || {Var, Spec1, Spec2} <- VarsWithSpec],
-	    emit(string:join(Options, " |~n      ") ++ ".~n"),
-	    emit("-type form() :: [property() | property_with_options() | xdata_field()].~n")
-    end.
+    Options = lists:flatmap(
+		fun(#xdata_field{type = T, var = Var} = F) ->
+			RecName = var_to_rec_field(Var, State),
+			Spec1 = mk_typespec(F, State),
+			Spec2 = case default(F, State) of
+				    undefined -> Spec1 ++ " | undefined";
+				    _ -> Spec1
+				end,
+			[io_lib:format("{'~s', ~s}", [RecName, Spec2])|
+			 if ?is_list_type(T) ->
+				 Spec3 = get_typespec(F, State),
+				 Spec4 = case is_complex_type(Spec3) of
+					     true ->
+						 io_lib:format("'~s'()", [RecName]);
+					     false ->
+						 Spec3
+					 end,
+				 [io_lib:format(
+				    "{'~s', ~s, options(~s)}",
+				    [RecName, Spec2, Spec4])];
+			    true ->
+				 []
+			 end]
+		end, Fs),
+    case lists:any(fun(#xdata_field{type = T}) -> ?is_list_type(T) end, Fs) of
+	true -> emit("-type options(T) :: [{binary(), T}].~n");
+	false -> ok
+    end,
+    emit("-type form_property() ::~n      "),
+    emit(string:join(Options, " |~n      ") ++ ".~n"),
+    emit("-type form() :: [form_property() | xdata_field()].~n~n"),
+    emit("-type error_reason() :: {form_type_mismatch, binary()} |~n"
+	 "                        {bad_var_value, binary(), binary()} |~n"
+	 "                        {missing_required_var, binary(), binary()} |~n"
+	 "                        {missing_value, binary(), binary()} |~n"
+	 "                        {too_many_values, binary(), binary()} |~n"
+	 "                        {unknown_var, binary(), binary()}.~n").
 
 mk_top_decoder(Fs, State) ->
     Required = [Var || #xdata_field{var = Var} <- Fs, is_required(Var, State)],
-    Guard = string:join(["XMLNS == <<\"" ++ NS ++ "\">>" || NS <- State#state.xmlns], "; "),
-    emit("decode(Fs) -> decode(Fs, []).~n"),
-    emit("decode(Fs, Acc) ->"
+    NSList = "[" ++ string:join(["<<\"" ++ NS ++ "\">>" || NS <- State#state.xmlns], ", ") ++ "]",
+    emit("decode(Fs) -> decode(Fs, ~s, ~p, []).~n",
+	 [NSList, Required]),
+    emit("decode(Fs, XMLNSList) -> decode(Fs, XMLNSList, ~p, []).~n",
+	 [Required]),
+    emit("decode(Fs, XMLNSList, Required) -> decode(Fs, XMLNSList, Required, []).~n"),
+    emit("decode(Fs, [_|_] = XMLNSList, Required, Acc) ->"
 	 "  case lists:keyfind(<<\"FORM_TYPE\">>, #xdata_field.var, Fs) of"
 	 "    false ->"
-	 "      decode(Fs, Acc, <<~p>>, ~p);"
-	 "    #xdata_field{values = [XMLNS]} when ~s ->"
-	 "      decode(Fs, Acc, XMLNS, ~p);"
-	 "    _ ->"
-	 "      erlang:error({?MODULE, {form_type_mismatch, <<~p>>}})~n"
-	 "  end.~n",
-	 [hd(State#state.xmlns), Required, Guard, Required,
-	  hd(State#state.xmlns)]).
+	 "      do_decode(Fs, hd(XMLNSList), Required, Acc);"
+	 "    #xdata_field{values = [XMLNS]} ->"
+	 "      case lists:member(XMLNS, XMLNSList) of"
+	 "          true -> do_decode(Fs, XMLNS, Required, Acc);"
+	 "          false -> erlang:error({?MODULE, {form_type_mismatch, XMLNS}})"
+	 "    end"
+	 "  end.~n").
 
 mk_top_encoder(Fs, State) ->
+    Required = "[" ++ string:join(
+			[io_lib:format("'~s'", [var_to_rec_field(Var, State)])
+			 || #xdata_field{var = Var} <- Fs, is_required(Var, State)],
+			", ") ++ "]",
     Clauses = string:join(
 		lists:map(
 		  fun(#xdata_field{var = Var, type = T}) when ?is_list_type(T) ->
 			  Field = var_to_rec_field(Var, State),
 			  io_lib:format(
-			    "{'~s', Val} -> ['encode_~s'(Val, default, Lang)];"
-			    "{'~s', Val, Opts} -> ['encode_~s'(Val, Opts, Lang)]",
-			    [Field, Field, Field, Field]);
+			    "{'~s', Val} ->"
+			    "  ['encode_~s'(Val, default, Lang, lists:member('~s', Required))];"
+			    "{'~s', Val, Opts} ->"
+			    "  ['encode_~s'(Val, Opts, Lang, lists:member('~s', Required))]",
+			    [Field, Field, Field, Field, Field, Field]);
 		     (#xdata_field{var = Var}) ->
 			  Field = var_to_rec_field(Var, State),
 			  io_lib:format(
-			    "{'~s', Val} -> ['encode_~s'(Val, Lang)];"
-			    "{'~s', _, _} -> erlang:error({badarg, Opt})",
+			    "{'~s', Val} -> ['encode_~s'(Val, Lang, lists:member('~s', Required))]",
 			    [Field, Field, Field])
-		  end, Fs) ++ ["#xdata_field{} -> [Opt]; _ -> []"],
+		  end, Fs) ++ ["#xdata_field{} -> [Opt]"],
 		";"),
     case State#state.xmlns of
 	[_] ->
-	    emit("encode(Cfg) -> encode(Cfg, <<\"en\">>).~n"),
-	    emit("encode(List, Lang) when is_list(List) ->");
+	    emit("encode(Cfg) -> encode(Cfg, <<\"en\">>, ~s).~n", [Required]),
+	    emit("encode(Cfg, Lang) -> encode(Cfg, Lang, ~s).~n", [Required]),
+	    emit("encode(List, Lang, Required) ->");
 	_ ->
-	    emit("encode(Cfg, XMLNS) -> encode(Cfg, XMLNS, <<\"en\">>).~n"),
-	    emit("encode(List, XMLNS, Lang) when is_list(List) ->")
+	    emit("encode(Cfg, XMLNS) -> encode(Cfg, XMLNS, <<\"en\">>, ~s).~n", [Required]),
+	    emit("encode(Cfg, XMLNS, Lang) -> encode(Cfg, XMLNS, Lang, ~s).~n", [Required]),
+	    emit("encode(List, XMLNS, Lang, Required) ->")
     end,
     XMLNS = case State#state.xmlns of
 		[NS] -> io_lib:format("<<~p>>", [NS]);
@@ -414,38 +454,33 @@ mk_decoder([#xdata_field{var = Var, type = Type} = F|Fs], State) ->
 		true ->
 		     mk_decoding_fun(F, State)
 	     end,
-    DelRequired = case is_required(Var, State) of
-		      true ->
-			  io_lib:format("lists:delete(~p, Required)", [Var]);
-		      false ->
-			  "Required"
-		  end,
+    DelRequired = io_lib:format("lists:delete(~p, Required)", [Var]),
     if Type == 'jid-multi' ->
 	    %% Psi work-around
-	    emit("decode([#xdata_field{var = ~p, values = [<<>>]} = F|Fs],"
-		 "       Acc, XMLNS, Required) ->~n"
+	    emit("do_decode([#xdata_field{var = ~p, values = [<<>>]} = F|Fs],"
+		 "       XMLNS, Required, Acc) ->~n"
 		 "    %% Psi work-around~n"
-		 "  decode([F#xdata_field{var = ~p, values = []}|Fs],"
-		 "         Acc, XMLNS, Required);",
+		 "  do_decode([F#xdata_field{var = ~p, values = []}|Fs],"
+		 "         XMLNS, Required, Acc);",
 		 [Var, Var]);
        true ->
 	    ok
     end,
-    emit("decode([#xdata_field{var = ~p, values = ~s}|Fs], Acc, XMLNS, Required) ->"
+    emit("do_decode([#xdata_field{var = ~p, values = ~s}|Fs], XMLNS, Required, Acc) ->"
 	 "  try ~s of"
-	 "    Result -> decode(Fs, [{'~s', Result}|Acc], XMLNS, ~s)"
+	 "    Result -> do_decode(Fs, XMLNS, ~s, [{'~s', Result}|Acc])"
 	 "  catch _:_ ->"
 	 "    erlang:error({?MODULE, {bad_var_value, ~p, XMLNS}})"
 	 "  end;",
-	 [Var, ValVar, DecFun, var_to_rec_field(Var, State),
-	  DelRequired, Var]),
+	 [Var, ValVar, DecFun, DelRequired,
+	  var_to_rec_field(Var, State), Var]),
     if not ?is_multi_type(Type) ->
-	    emit("decode([#xdata_field{var = ~p, values = []} = F|Fs],"
-		 "       Acc, XMLNS, Required) ->"
-		 "  decode([F#xdata_field{var = ~p, values = [<<>>]}|Fs],"
-		 "         Acc, XMLNS, Required);",
+	    emit("do_decode([#xdata_field{var = ~p, values = []} = F|Fs],"
+		 "       XMLNS, Required, Acc) ->"
+		 "  do_decode([F#xdata_field{var = ~p, values = [<<>>]}|Fs],"
+		 "         XMLNS, Required, Acc);",
 		 [Var, Var]),
-	    emit("decode([#xdata_field{var = ~p}|_], _, XMLNS, _) ->"
+	    emit("do_decode([#xdata_field{var = ~p}|_], XMLNS, _, _) ->"
 		 "  erlang:error({?MODULE, {too_many_values, ~p, XMLNS}});",
 		 [Var, Var]);
        true ->
@@ -453,52 +488,67 @@ mk_decoder([#xdata_field{var = Var, type = Type} = F|Fs], State) ->
     end,
     mk_decoder(Fs, State);
 mk_decoder([], State) ->
-    emit("decode([#xdata_field{var = Var}|Fs], Acc, XMLNS, Required) ->"
-	 "  if Var /= <<\"FORM_TYPE\">> ->"
-	 "    erlang:error({?MODULE, {unknown_var, Var, XMLNS}});"
-	 "  true ->"
-	 "    decode(Fs, Acc, XMLNS, Required)"
-	 "  end;"),
-    if State#state.required /= [] ->
-	    emit("decode([], _, XMLNS, [Var|_]) ->"
-		 "  erlang:error({?MODULE, {missing_required_var, Var, XMLNS}});~n");
+    if State#state.ignore_unknown ->
+	    emit("do_decode([_|Fs], XMLNS, Required, Acc) ->"
+		 "  do_decode(Fs, XMLNS, Required, Acc);");
        true ->
-	    ok
+	    emit("do_decode([#xdata_field{var = Var}|Fs], XMLNS, Required, Acc) ->"
+		 "  if Var /= <<\"FORM_TYPE\">> ->"
+		 "    erlang:error({?MODULE, {unknown_var, Var, XMLNS}});"
+		 "  true ->"
+		 "    do_decode(Fs, XMLNS, Required, Acc)"
+		 "  end;")
     end,
-    emit("decode([], Acc, _, []) -> Acc.~n").
+    emit("do_decode([], XMLNS, [Var|_], _) ->"
+	 "  erlang:error({?MODULE, {missing_required_var, Var, XMLNS}});~n"),
+    emit("do_decode([], _, [], Acc) -> Acc.~n").
 
 mk_encoders(Fs, State) ->
     lists:foreach(
-      fun(#xdata_field{var = Var, required = IsRequired, desc = Desc,
+      fun(#xdata_field{var = Var, desc = Desc,
 		       label = Label, type = Type} = F) ->
+	      Spec0 = mk_typespec(F, State),
+	      Spec1 = case default(F, State) of
+			  undefined -> Spec0 ++ " | undefined";
+			  _ -> Spec0
+		      end,
 	      EncVals = mk_encoded_values(F, State),
 	      EncOpts = mk_encoded_options(F, State),
 	      FieldName = var_to_rec_field(Var, State),
 	      DescStr = if Desc == <<>> -> "<<>>";
-			   true -> io_lib:format("xmpp_tr:tr(Lang, ~p)", [Desc])
+			   true -> io_lib:format("xmpp_tr:tr(Lang, ?T(\"~s\"))", [Desc])
 			end,
 	      LabelStr = if Label == <<>> -> "<<>>";
-			    true -> io_lib:format("xmpp_tr:tr(Lang, ~p)", [Label])
+			    true -> io_lib:format("xmpp_tr:tr(Lang, ?T(\"~s\"))", [Label])
 			 end,
 	      if ?is_list_type(Type) ->
-		      emit("'encode_~s'(Value, Options, Lang) ->", [FieldName]);
+		      Spec2 = get_typespec(F, State),
+		      Spec3 = case is_complex_type(Spec2) of
+				  true -> io_lib:format("'~s'()", [FieldName]);
+				  false -> Spec2
+			      end,
+		      emit("-spec 'encode_~s'(~s, default | options(~s), binary(), boolean()) -> xdata_field().~n"
+			   "'encode_~s'(Value, Options, Lang, IsRequired) ->",
+			   [FieldName, Spec1, Spec3, FieldName]);
 		 true ->
-		      emit("'encode_~s'(Value, Lang) ->", [FieldName])
+		      emit("-spec 'encode_~s'(~s, binary(), boolean()) -> xdata_field().~n"
+			   "'encode_~s'(Value, Lang, IsRequired) ->",
+			   [FieldName, Spec1, FieldName])
 	      end,
 	      emit("  Values = ~s,"
 		   "  Opts = ~s,"
 		   "  #xdata_field{var = ~p,"
 		   "               values = Values,"
-		   "               required = ~p,"
+		   "               required = IsRequired,"
 		   "               type = ~p,"
 		   "               options = Opts,"
 		   "               desc = ~s,"
 		   "               label = ~s}.~n",
-		   [EncVals, EncOpts, Var, IsRequired, Type, DescStr, LabelStr])
+		   [EncVals, EncOpts, Var, Type, DescStr, LabelStr])
       end, Fs).
 
 mk_encoded_values(#xdata_field{var = Var, type = Type,
-			       options = Options}, State) ->
+			       options = Options} = F, State) ->
     EncFun =
 	case get_enc_fun(Var, Type, Options, State) of
 	    {M, Fun, Args} ->
@@ -513,20 +563,17 @@ mk_encoded_values(#xdata_field{var = Var, type = Type,
 			"[" ++ io_lib:format("~s~s(Value~s)", [Mod, Fun, FArgs])
 			    ++ "]"
 		end;
+	    undefined when ?is_multi_type(Type) ->
+		"Value";
 	    undefined ->
 		"[Value]"
 	end,
-    Default = case get_dec_fun(Var, Type, Options, State) of
-		  _ when ?is_multi_type(Type) -> "[]";
-		  undefined -> "<<>>";
-		  _MFA -> "undefined"
-	      end,
     io_lib:format(
       "case Value of"
-      "  ~s -> [];~n"
+      "  ~p -> [];~n"
       "  Value -> ~s~n"
       "end",
-      [Default, EncFun]).
+      [default(F, State), EncFun]).
 
 mk_encoded_options(#xdata_field{var = Var, type = Type,
 				options = Options}, State) ->
@@ -546,7 +593,7 @@ mk_encoded_options(#xdata_field{var = Var, type = Type,
 			 io_lib:format("#xdata_option{value = ~p}", [V]);
 		     _ ->
 			 io_lib:format(
-			   "#xdata_option{label = xmpp_tr:tr(Lang, ~p), value = ~p}",
+			   "#xdata_option{label = xmpp_tr:tr(Lang, ?T(\"~s\")), value = ~p}",
 			   [L, V])
 		 end || #xdata_option{label = L, value = V} <- Options],
 		","),
@@ -607,6 +654,8 @@ get_dec_fun(Var, Type, Options, State) ->
 	    {M, F, A};
 	{Var, {dec_bool, []}} ->
 	    {undefined, dec_bool, []};
+	{Var, {dec_ip, []}} ->
+	    {undefined, dec_ip, []};
 	{Var, {not_empty, []}} ->
 	    {undefined, not_empty, []};
 	{Var, {dec_enum, [Variants]}} ->
@@ -627,6 +676,8 @@ get_enc_fun(Var, Type, Options, State) ->
 	    {undefined, enc_int, []};
 	{undefined, dec_enum_int, _} ->
 	    {undefined, enc_enum_int, []};
+	{undefined, dec_ip, _} ->
+	    {undefined, enc_ip, []};
 	{jid, decode, []} ->
 	    {jid, encode, []};
 	_ ->
@@ -644,7 +695,9 @@ get_enc_fun(Var, Type, Options, State) ->
 		{Var, {enc_int, _}} ->
 		    {undefined, enc_int, []};
 		{Var, {dec_enum_int, _}} ->
-		    {undefined, enc_enum_int, []}
+		    {undefined, enc_enum_int, []};
+		{Var, {dec_ip, _}} ->
+		    {undefined, enc_ip, []}
 	    end
     end.
 
@@ -668,6 +721,8 @@ get_typespec(#xdata_field{var = Var, type = Type, options = Options}, State) ->
 		    enum_spec(Args);
 		{undefined, dec_bool, _} ->
 		    "boolean()";
+		{undefined, dec_ip, _} ->
+		    "inet:ip_address()";
 		{jid, decode, _} ->
 		    "jid:jid()";
 		{undefined, dec_int, Args} ->
@@ -718,3 +773,27 @@ normalize(Txt) when is_binary(Txt) ->
 	T ->
 	    list_to_binary(string:join(T, " "))
     end.
+
+default(#xdata_field{type = Type}, _) when ?is_multi_type(Type) ->
+    [];
+default(#xdata_field{var = Var, type = Type,
+		     options = Options}, State) ->
+    case get_dec_fun(Var, Type, Options, State) of
+	undefined -> <<>>;
+	_MFA -> undefined
+    end.
+
+codec_funs() ->
+    [{dec_int, 3},
+     {dec_int, 1},
+     {dec_enum, 2},
+     {dec_enum_int, 2},
+     {dec_enum_int, 4},
+     {enc_int, 1},
+     {enc_enum, 1},
+     {enc_enum_int, 1},
+     {not_empty, 1},
+     {dec_bool, 1},
+     {enc_bool, 1},
+     {dec_ip, 1},
+     {enc_ip, 1}].
